//
// ARF - Augmented Reality Framework (ETSI ISG ARF)
//
// Copyright 2022 ETSI
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Last change: June 2022
//

/*
 * World Storage API
 *
 * API ensuring interoperability between an authoring tool and a World Storage service
 *
 * The version of the OpenAPI document: 1.0.0
 * 
 * Generated by: https://openapi-generator.tech
 */

using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Http;
using Swashbuckle.AspNetCore.Annotations;
using Swashbuckle.AspNetCore.SwaggerGen;
using Newtonsoft.Json;
using Org.OpenAPITools.Attributes;
using Org.OpenAPITools.Models;
using Org.OpenAPITools.Services;
using MongoDB.Driver;

namespace Org.OpenAPITools.Controllers
{
  /// <summary>
  /// 
  /// </summary>
  [ApiController]
  public class WorldLinksApiControllerImpl : WorldLinksApiController
  {

    private readonly WorldLinkService _worldLinkService;

    /// <summary>
    /// 
    /// </summary>
    public WorldLinksApiControllerImpl(WorldLinkService worldLinkService)
    {
      _worldLinkService = worldLinkService;
    }

    /// <summary>
    /// Create a link between world anchors and trackables.
    /// </summary>
    /// <param name="worldLink">The link to be added to the world storage.</param>
    /// <response code="200">OK, returns the UUID of the link defined by the world storage.</response>
    /// <response code="201">Null response</response>
    /// <response code="400">Bad request.</response>
    /// <response code="409">Invalid UUID, id must be a Nil value.</response>
    /// <response code="0">Unexpected error.</response>
    [HttpPost]
    [Route("/worldLinks")]
    [Consumes("application/json")]
    [ValidateModelState]
    [SwaggerOperation("AddWorldLink")]
    [SwaggerResponse(statusCode: 200, type: typeof(string), description: "OK, returns the UUID of the link defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "Unexpected error.")]
    public override IActionResult AddWorldLink([FromBody] WorldLink worldLink)
    {
      if (String.IsNullOrEmpty(worldLink.UUID.ToString()))
      {
        worldLink.UUID = Guid.NewGuid();
      }
      try
      {
        WorldLink myworldlink = _worldLinkService.Create(worldLink);
        return new ObjectResult(myworldlink);
      }
      catch (Exception e)
      {
        return StatusCode(400, e.Message);
      }
    }

    /// <summary>
    /// Deletes a worldLink.
    /// </summary>
    /// <param name="worldLinkUUID">link id to delete</param>
    /// <response code="200">OK</response>
    /// <response code="400">Invalid UUID supplied.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    [HttpDelete]
    [Route("/worldLinks/{worldLinkUUID}")]
    [ValidateModelState]
    [SwaggerOperation("DeleteWorldLink")]
    public override IActionResult DeleteWorldLink([FromRoute(Name = "worldLinkUUID")][Required] Guid worldLinkUUID)
    {
      DeleteResult answer = _worldLinkService.Remove((worldLinkUUID));
      return (answer.IsAcknowledged && answer.DeletedCount > 0) ? new ObjectResult("ok") : StatusCode(404, "Not found, could not find UUID in database.");
    }


    /// <summary>
    /// Find a link by its UUID.
    /// </summary>
    /// <param name="worldLinkUUID">UUID of the link to retrieve.</param>
    /// <response code="200">Successful operation.</response>
    /// <response code="400">Invalid UUID supplied.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    [HttpGet]
    [Route("/worldLinks/{worldLinkUUID}")]
    [ValidateModelState]
    [SwaggerOperation("GetWorldLinkById")]
    [SwaggerResponse(statusCode: 200, type: typeof(WorldLink), description: "Successful operation.")]
    public override IActionResult GetWorldLinkById([FromRoute(Name = "worldLinkUUID")][Required] Guid worldLinkUUID)
    {
      WorldLink myworldlink = _worldLinkService.Get(worldLinkUUID);
      if (null != myworldlink)
      {
        // check TypeFrom
        if (myworldlink.TypeFrom == ObjectType.TrackableEnum)
        {
          if (null == _worldLinkService.GetTrackable(myworldlink.UUIDFrom))
          {
            myworldlink.TypeFrom = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDFrom = Guid.Empty;
          }
        }
        else if (myworldlink.TypeFrom == ObjectType.WorldAnchorEnum)
        {
          if (null == _worldLinkService.GetAnchor(myworldlink.UUIDFrom))
          {
            myworldlink.TypeFrom = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDFrom = Guid.Empty;
          }
        }
        // check TypeTo
        if (myworldlink.TypeTo == ObjectType.TrackableEnum)
        {
          if (null == _worldLinkService.GetTrackable(myworldlink.UUIDTo))
          {
            myworldlink.TypeTo = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDTo = Guid.Empty;
          }
        }
        else if (myworldlink.TypeTo == ObjectType.WorldAnchorEnum)
        {
          if (null == _worldLinkService.GetAnchor(myworldlink.UUIDTo))
          {
            myworldlink.TypeTo = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDTo = Guid.Empty;
          }
        }
      }
      return (null != myworldlink) ? new ObjectResult(myworldlink) : StatusCode(404, "Not found, could not find UUID in database.");
    }

    /// <summary>
    /// Returns the list of all links defined by the world storage.
    /// </summary>
    /// <response code="200">OK returns all the worldLinks defined by the world storage.</response>
    /// <response code="201">Null response</response>
    /// <response code="0">Unexpected error.</response>
    [HttpGet]
    [Route("/worldLinks")]
    [ValidateModelState]
    [SwaggerOperation("GetWorldLinks")]
    [SwaggerResponse(statusCode: 200, type: typeof(List<WorldLink>), description: "OK returns all the worldLinks defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "Unexpected error.")]
    public override IActionResult GetWorldLinks()
    {
      List<WorldLink> worldlinklist = _worldLinkService.Get();
      foreach (WorldLink myworldlink in worldlinklist)
      {
        // check TypeFrom
        if (myworldlink.TypeFrom == ObjectType.TrackableEnum)
        {
          if (null == _worldLinkService.GetTrackable(myworldlink.UUIDFrom))
          {
            myworldlink.TypeFrom = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDFrom = Guid.Empty;
          }
        }
        else if (myworldlink.TypeFrom == ObjectType.WorldAnchorEnum)
        {
          if (null == _worldLinkService.GetAnchor(myworldlink.UUIDFrom))
          {
            myworldlink.TypeFrom = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDFrom = Guid.Empty;
          }
        }
        // check TypeTo
        if (myworldlink.TypeTo == ObjectType.TrackableEnum)
        {
          if (null == _worldLinkService.GetTrackable(myworldlink.UUIDTo))
          {
            myworldlink.TypeTo = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDTo = Guid.Empty;
          }
        }
        else if (myworldlink.TypeTo == ObjectType.WorldAnchorEnum)
        {
          if (null == _worldLinkService.GetAnchor(myworldlink.UUIDTo))
          {
            myworldlink.TypeTo = ObjectType.NotIdentifiedEnum;
            myworldlink.UUIDTo = Guid.Empty;
          }
        }
      }
      return new ObjectResult(worldlinklist);
    }



    /// <summary>
    /// Modify a World Link.
    /// </summary>
    /// <remarks>Modify an existing World Link given a json object containing all the required informations. &lt;br&gt; **Please note that ID of the object is required in the JSON**</remarks>
    /// <param name="worldLink">The World Link to be modified in the world storage.</param>
    /// <response code="200">OK, return the UUID of the modified World Link.</response>
    /// <response code="400">Bad request.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    /// <response code="0">Unexpected error.</response>
    [HttpPut]
    [Route("/worldLinks")]
    [Consumes("application/json")]
    [ValidateModelState]
    [SwaggerOperation("ModifyWorldLink")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "Unexpected error.")]
    public override IActionResult ModifyWorldLink([FromBody] WorldLink worldLink)
    {
      ReplaceOneResult result = _worldLinkService.Update(worldLink.UUID, worldLink);
      if (result.MatchedCount == 0)
      {
        return StatusCode(404, "Not found, could not find UUID in database.");
      }
      else
      {
        return StatusCode(200, worldLink.UUID);
      }
    }
  }
}
